/*
 *
 * $Id$
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
#else
 #ifdef HAVE_SYS_TIME_H
  #include <sys/time.h>
 #else
  #include <time.h>
 #endif
#endif
#include <errno.h>

#include "alist.h"
#include "img.h"
#include "pool.h"
#include "sentry.h"
#include "client.h"

typedef struct {
    char magick[16];
    int32_t ver;
    int32_t len;
} token_t;


#define MAX_IMGSIZE     (128*1024)

/*
 * in file, images are recorded with network order
 * you have to convert them to host order.
 */


/*
 * fname    source file path
 * form     destination file path (e.g., /var/tmp/i%04d.jpg)
 * rc       record counter
 */

int
parse_split_sentry_packedfile(char *fname, char *form, int *rc)
{
    unsigned char tokenbuf[BUFSIZ];
    int chk;
    int fd;
    off_t pos;
    token_t rtoken, ctoken;
    int skiplen;
    int nr;
    int tokenlen;
    int32_t *p;
    unsigned char headbuf[BUFSIZ];
    sentry_imgrec rtmpimg, ctmpimg;
    unsigned char *body_ptr;
    int rc_start;

    tokenlen = 6 + sizeof(int32_t)*2;
    rc_start = *rc;

    if(debug>=100) {
        fprintf(stderr, "reading target path '%s'\n", fname);
    }

    fd = open(fname, O_RDONLY);
    if(fd<0) {
        fprintf(stderr, "fail open '%s' (%d)\n", fname, errno);
        goto fail;
    }

    pos = lseek(fd, 0, SEEK_SET);
    if(pos<0) {
        fprintf(stderr, "fail lseek '%s' (%d)\n", fname, errno);
        goto fail;
    }

/*
    rc = 0;
*/
    while(1) {
        memset(tokenbuf, 0xff, tokenlen);

try_read1:
        nr = read(fd, tokenbuf, tokenlen);
        if(nr<0) {
            fprintf(stderr, "fail read '%s' (%d)\n", fname, errno);
            goto suspend;
        }
        if(nr==0) {
            if(debug>=100)
            fprintf(stderr, "       done\n");
            break;
        }
        if(nr<tokenlen) {
            fprintf(stderr, "      lack of size, read %d expect %d. give up\n",
                nr, tokenlen);
            break;
        }

        if(debug>=2000) {
            int i;
            int v;

            fprintf(stderr, "   ");
            for(i=0;i<nr;i++) {
                fprintf(stderr, "%02X ", i);
            }
            fprintf(stderr, " \n");
            
            fprintf(stderr, "  <");
            for(i=0;i<nr;i++) {
                fprintf(stderr, "%02x ", tokenbuf[i]);
            }
            fprintf(stderr, ">\n");
        }

        memset(&rtoken, 0, sizeof(rtoken));

        strncpy(rtoken.magick,  &tokenbuf[0],  6);
        memcpy(&rtoken.ver,   &tokenbuf[6],  4);
        memcpy(&(rtoken.len),   &tokenbuf[10], sizeof(int32_t));

#if 1
        fprintf(stderr, " %4d: <%06d> r \"%c%c%c%c%c%c\" %2d %4d\n",
            *rc, (long)pos,

            rtoken.magick[0],
            rtoken.magick[1],
            rtoken.magick[2],
            rtoken.magick[3],
            rtoken.magick[4],
            rtoken.magick[5],
            rtoken.ver, rtoken.len);
#endif



        chk = strncmp(rtoken.magick, "SENTRY", 6);
        if(chk!=0) {
            fprintf(stderr, "ignore magic '%c%c%c%c%c' %d\n",
                rtoken.magick[0],
                rtoken.magick[1],
                rtoken.magick[2],
                rtoken.magick[3],
                rtoken.magick[4],
                rtoken.magick[6],
                chk);
            goto suspend;
        }

        strncpy(ctoken.magick, rtoken.magick, 6);
        ctoken.ver = ntohl(rtoken.ver);
        ctoken.len = ntohl(rtoken.len);
    
        skiplen = ctoken.len;

#if 1
        fprintf(stderr, " %4d: <%06d> c \"%c%c%c%c%c%c\" %2d %4d\n",
            *rc, (long)pos,
            ctoken.magick[0],
            ctoken.magick[1],
            ctoken.magick[2],
            ctoken.magick[3],
            ctoken.magick[4],
            ctoken.magick[5],
            ctoken.ver, ctoken.len);
#endif

try_read2:
#if 0
        nr = read(fd, headbuf, sizeof(sentry_imgrec));
#endif
        nr = read(fd, headbuf, SENTRY_SHM_HEADER_SIZE);
        if(nr<0) {
            if(errno==EINTR) {
                goto try_read2;
            }
            fprintf(stderr, "cannot read header (%d)\n", errno);
            goto suspend;
        }
        if(nr==0) {
            break;
        }

        skiplen -= nr;
        
        memcpy(&rtmpimg, headbuf, sizeof(sentry_imgrec));
#if 0
        /* unreadble data (because it is not host order) */
        if(debug>=100)
            show_imgattr(stderr, &rtmpimg);
#endif

        ntoh_img(&ctmpimg, &rtmpimg);
        if(debug>=100)
            show_imgattr(stderr, &ctmpimg);

        if(ctmpimg.length>MAX_IMGSIZE) {
            body_ptr = NULL;

            ctmpimg.body_len = 0;
            ctmpimg.body = NULL;
        }
        else {
            body_ptr = (unsigned char*)malloc(ctmpimg.length);
try_bodyread1:
            nr = read(fd, body_ptr, ctmpimg.length);
            if(nr<0) {
                if(errno==EINTR) {
                    goto try_bodyread1;
                }
                fprintf(stderr, "cannot read header (%d)\n", errno);
                goto suspend;
            }

#if 0
            fprintf(stderr, "read body %d\n", ctmpimg.length);
#endif
            skiplen -= ctmpimg.length;

            ctmpimg.body_len = ctmpimg.length;
            ctmpimg.body = body_ptr;

 
 {
            char tmppath[BUFSIZ];

            /* 30frame/sec * 60sec/min = 1800frame/min */
/*
            sprintf(tmppath, "/var/tmp/i%04d.jpg", *rc);
*/
            sprintf(tmppath, form, *rc);

            dump_to_file(tmppath, ctmpimg.body, ctmpimg.length);
 }
            free(body_ptr);
            body_ptr = NULL;
        }

#if 0
        fprintf(stderr, "  r length %d, size %dx%d\n",
            ntohl(rtmpimg.length),
            ntohl(rtmpimg.width), ntohl(rtmpimg.height));
#endif


#if 0
        fprintf(stderr, " %4d: <%06d> c \"%c%c%c%c%c%c\" %2d %4d\n",
            *rc, (long)pos,
            ctoken.magick[0],
            ctoken.magick[1],
            ctoken.magick[2],
            ctoken.magick[3],
            ctoken.magick[4],
            ctoken.magick[5],
            ctoken.ver, ctoken.len);
#endif
#if 0
        fprintf(stderr, "  c length %d, size %dx%d\n",
            (ctmpimg.length),
            (ctmpimg.width), (ctmpimg.height));
#endif


        if(debug>=100) 
        fprintf(stderr, " %4d: %d.%06d %d.%06d %s %s %03dx%03d\n",
            *rc,
            ctmpimg.send_sec, ctmpimg.send_usec,
            ctmpimg.recv_sec, ctmpimg.recv_usec,
            assoc_v(format_alist, ctmpimg.format),
            assoc_v(color_alist, ctmpimg.corder),
            ctmpimg.width, ctmpimg.height);
    

        if(debug>=100)
            show_imgattr(stderr, &ctmpimg);


        (*rc)++;

        if(skiplen) {
            pos = lseek(fd, skiplen, SEEK_CUR);
            if(pos<0) {
                fprintf(stderr, "fail lseek '%s' (%d)\n", fname, errno);
                break;
            }
        }
    }

suspend:
    close(fd);

    if(debug>=50)
        fprintf(stderr, "%4d images [%4d,%4d) are stored in '%s'\n",
            *rc-rc_start, rc_start, *rc, fname);

    
    goto out;
    
fail:

out:
    return 0;
}

main(int argc, char **argv)
{
    int i;
    int nc;

    debug=1000;
    
    nc = 0;
    for(i=1;i<argc;i++) {
        parse_split_sentry_packedfile(argv[i], "/var/tmp/i%04d.jpg", &nc);
    }
    if(debug)
        fprintf(stderr, "%d images are sweeped\n", nc);
}

