/*
 * $Id$
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
/*
#include <string.h>
#include <math.h>
*/
#ifdef TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
#else
 #ifdef HAVE_SYS_TIME_H
  #include <sys/time.h>
 #else
  #include <time.h>
 #endif
#endif
#include <errno.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>

#include <X11/Xmu/WinUtil.h>

#include "hook.h"
#include "alist.h"
#include "sentry.h"

int SetupCmap(Display *dpy, Colormap cmap);

extern  int output_format;
extern  int output_corder;
   

void
ListVisual(Display *dpy, FILE* ofp)
{
    XVisualInfo *vlist;
    int num_vlist;
    int i;

    fprintf(ofp, "___\n");

    fprintf(ofp, "%3s: %-8s %3s %3s %3s %-12s %5s %8s %7s\n",
        "#", "address", "dec", "hex", "scr", "class", "depth",
        "color", "bit/rgb");

    vlist = XGetVisualInfo(dpy, VisualNoMask, NULL, &num_vlist);
    for(i=0;i<num_vlist;i++) {
        fprintf(ofp, "%3d: %p %3d %3x %3d %-12s %2d %4d %2d %8x %8x %8x\n",
            i,
            vlist[i].visual,
            vlist[i].visualid,
            vlist[i].visualid,
            vlist[i].screen,
            (vlist[i].class==DirectColor) ? "DirectColor"
              : ( (vlist[i].class==TrueColor) ? "TrueColor"
                : ( (vlist[i].class==PseudoColor) ? "PseudoColor"
                  : ( (vlist[i].class==StaticColor) ? "StaticColor"
                    : ( (vlist[i].class==GrayScale) ? "GrayScale"
                      : ( (vlist[i].class==StaticGray) ? "StaticGray"
                        : "Uknown"
                        )
                      )
                    )
                  )
                )
            ,
            vlist[i].depth,
            vlist[i].colormap_size,
            vlist[i].bits_per_rgb,
            vlist[i].visual->red_mask,
            vlist[i].visual->green_mask,
            vlist[i].visual->blue_mask
            );
    }
    XFree(vlist);

    fprintf(ofp, "___\n");
}





extern Visual   *xp_slide_visual,   *xp_foil_visual;
extern int      xp_slide_depth,     xp_foil_depth;
extern Colormap xp_slide_cmap,      xp_foil_cmap;
extern int      xp_use_special_cmap;



/*
 *
 * This routine sets visual, depth and cmap automatically.
 * In TrueColor, cmaps are made with AlloNone.
 * In PseudoColor, cmaps are made with AllocAll, and these are independented,
 * because PseudoColor's color space is very small.
 *
 * POLICY:
 *      Since foil window is appeared over others windows, foil cmap will
 *      be shared among other windows.  Therefore, default cmap uses as foil
 *      cmap in PseudoColor.
 *
 */

int SetupVisual(Display *dpy, String fvisual, String svisual)
{
    XVisualInfo *vlist;
    int ninfo;
    XVisualInfo vinfo;
    Visual *Ptrue, *Ppseudo;
    int truedepth, pseudodepth;

    xp_use_special_cmap = 0;

    TraceOff;
    
    if(debug>=10)
        ListVisual(dpy, stderr);

    Ptrue = NULL;
    Ppseudo = NULL;

    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 24, TrueColor, &vinfo)) {
        Ptrue = vinfo.visual;
        truedepth = 24;
    }
    else
    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 16, TrueColor, &vinfo)) {
        Ptrue = vinfo.visual;
        truedepth = 16;
    }
    else
    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 15, TrueColor, &vinfo)) {
        Ptrue = vinfo.visual;
        truedepth = 15;
    }
    else
    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 12, TrueColor, &vinfo)) {
        Ptrue = vinfo.visual;
        truedepth = 12;
    }
#if 0
    else
    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 8, TrueColor, &vinfo)) {
        Ptrue = vinfo.visual;
        truedepth = 8;
    }
#endif

    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 12, PseudoColor, &vinfo)) {
        Ppseudo = vinfo.visual;
        pseudodepth = 12;
    }
    else
    if(XMatchVisualInfo(dpy,
            DefaultScreen(dpy), 8, PseudoColor, &vinfo)) {
        Ppseudo = vinfo.visual;
        pseudodepth = 8;
    }

#if 1

    if(Ptrue!=NULL) {
        if(Ptrue->class == TrueColor) {
            Trace("found TrueColor visual id %d (%#x) and correct %d\n",
                Ptrue->visualid, Ptrue->visualid, Ptrue->class);
        }
        else {
            Trace("found TrueColor visual id %d (%#x), but strange %d\n",
                Ptrue->visualid, Ptrue->visualid, Ptrue->class);
        }
    }
    else {
        Trace("*no* TrueColor\n");
    }

    if(Ppseudo!=NULL) {
        if(Ppseudo->class == PseudoColor) {
            Trace("found PseudoColor visual id %d (%#x) and correct %d\n",
                Ppseudo->visualid, Ppseudo->visualid, Ppseudo->class);
        }
        else {
            Trace("found PseudoColor visual id %d (%#x), but strange %d\n",
                Ppseudo->visualid, Ppseudo->visualid, Ppseudo->class);
        }
    }
    else {
        Trace("*no* PseudoColor\n");
    }

#endif

    if(fvisual==NULL && svisual==NULL) {
        Trace("foil visual is automatic, slide visual is automatic, too.\n");


        if(Ptrue!=NULL) {
            Trace("use TrueColor\n");

            xp_slide_visual = xp_foil_visual = Ptrue;
            xp_slide_depth  = xp_foil_depth  = truedepth;
            xp_slide_cmap   = xp_foil_cmap   = XCreateColormap(dpy,
                    DefaultRootWindow(dpy),
                    xp_slide_visual, AllocNone);
#if 1
            if(truedepth==24) {
                if(Ptrue->red_mask==0xff) {
                    output_corder = SEN_C_XBGR;
                }
                else
                if(Ptrue->blue_mask==0xff) {
                    output_corder = SEN_C_XRGB;
                }
                if(debug) {
                    fprintf(stderr, "output_corder %s(%d)\n",
                        assoc_v(&color_alist, output_corder), output_corder);
                }
            }
            
#endif
        }
        else
        if(Ppseudo!=NULL) {
            Trace("use PseudoColor\n");

            xp_slide_visual = xp_foil_visual = Ppseudo;
            xp_slide_depth  = xp_foil_depth  = pseudodepth;
            xp_slide_cmap   = XCreateColormap(dpy,
                    DefaultRootWindow(dpy),
                    xp_slide_visual, AllocAll);
#if 0
            xp_foil_cmap   = XCreateColormap(dpy,
                    DefaultRootWindow(dpy),
                    xp_foil_visual, AllocAll);
#endif
            xp_foil_cmap = DefaultColormap(dpy, DefaultScreen(dpy));
        }
        else {
            Trace("not found support visual\n");
            Trace("    this program supports TrueColor and PseudoColor\n");
        }

    }
    else {

        if(svisual!=NULL) {
            Trace("specified visual '%s'(%d) for slide\n", svisual, atoi(svisual));
            vinfo.visualid = atoi(svisual);
            vlist = XGetVisualInfo(dpy, VisualIDMask, &vinfo, &ninfo);
            if(vlist==NULL) {
                Trace("specified visual '%s'(%d) for slide is not found\n",
                    svisual, atoi(svisual));
                exit(4);
            }
            xp_slide_visual = vlist[0].visual;
            xp_slide_depth = vlist[0].depth;
            if(xp_slide_visual->class==TrueColor) {
                xp_slide_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_slide_visual, AllocNone);
            }
            else
            if(xp_slide_visual->class==PseudoColor) {
                xp_slide_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_slide_visual, AllocAll);
            }
        }
        else {
            Trace("slide visual is setted automatic\n");
            if(Ptrue!=NULL) {
                Trace("found TrueColor\n");

                xp_slide_visual = Ptrue;
                xp_slide_depth  = truedepth;
                xp_slide_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_slide_visual, AllocNone);
            }
            else
            if(Ppseudo!=NULL) {
                Trace("found PseudoColor\n");

                xp_slide_visual = Ppseudo;
                xp_slide_depth  = pseudodepth;
                xp_slide_cmap = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_slide_visual, AllocAll);
            }
            else {
                Trace("not found support visual for slide\n");
                Trace("    this program supports TrueColor and PseudoColor\n");
            }
        }


        if(fvisual!=NULL) {
            Trace("specified visual '%s'(%d) for foil\n", fvisual, atoi(fvisual));
            vinfo.visualid = atoi(fvisual);
            vlist = XGetVisualInfo(dpy, VisualIDMask, &vinfo, &ninfo);
            if(vlist==NULL) {
                Trace("specified visual '%s'(%d) for foil is not found\n",
                    fvisual, atoi(fvisual));
                exit(4);
            }
            xp_foil_visual = vlist[0].visual;
            xp_foil_depth = vlist[0].depth;
            if(xp_foil_visual->class==TrueColor) {
                xp_foil_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_foil_visual, AllocNone);
            }
            else
            if(xp_foil_visual->class==PseudoColor) {
#if 0
                xp_foil_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_foil_visual, AllocAll);
#endif
                xp_foil_cmap = DefaultColormap(dpy, DefaultScreen(dpy));
            }

        }
        else {
            Trace("foil visual is setted automatic\n");
            if(Ptrue!=NULL) {
                Trace("found TrueColor\n");

                xp_foil_visual = Ptrue;
                xp_foil_depth  = truedepth;
                xp_foil_cmap   = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_foil_visual, AllocNone);
            }
            else
            if(Ppseudo!=NULL) {
                Trace("found PseudoColor\n");

                xp_foil_visual = Ppseudo;
                xp_foil_depth  = pseudodepth;
#if 0
                xp_foil_cmap = XCreateColormap(dpy,
                        DefaultRootWindow(dpy),
                        xp_foil_visual, AllocAll);
#endif
                xp_foil_cmap = DefaultColormap(dpy, DefaultScreen(dpy));
            }
            else {
                Trace("not found support visual for foil\n");
                Trace("    this program supports TrueColor and PseudoColor\n");
            }
        }

    }


    Trace("foil:  visual id %d (%#x), depth %2d, cmap %d (%#x)\n",
        xp_foil_visual->visualid, xp_foil_visual->visualid,
            xp_foil_depth, xp_foil_cmap, xp_foil_cmap);
    Trace("slide: visual id %d (%#x), depth %2d, cmap %d (%#x)\n",
        xp_slide_visual->visualid, xp_slide_visual->visualid,
            xp_slide_depth, xp_slide_cmap, xp_slide_cmap);
}






